package tnntpd;

import java.util.Date;
import java.util.LinkedList;
import java.sql.*;

public class Group {

  private int id ;
  private String name ;
  private Timestamp creationDate = new Timestamp (new Date().getTime());
  private int firstArticle = 0;
  private int lastArticle = -1;
  private Timestamp lastArticleDate = null;
  
  public static Group getById ( int id ) {
    try {
      PreparedStatement stmt = NNTPConnection.getDB().prepareStatement("SELECT * FROM groups WHERE id = ?;") ;
      stmt.setInt(1,id) ;
      return loadFromStatement ( stmt ) ;
    } catch ( SQLException e ) {
      return null ;
    }
  }

  public static Group getByName ( String name ) {
    try {
      PreparedStatement stmt = NNTPConnection.getDB().prepareStatement("SELECT * FROM groups WHERE name = ?;") ;
      stmt.setString(1,name) ;
      return loadFromStatement ( stmt ) ;
    } catch ( SQLException e ) {
      return null ;
    }
  }

  public static Group loadFromStatement ( PreparedStatement stmt ) {
    try {
      ResultSet rs = stmt.executeQuery() ;
      rs.next() ;
      return loadFromResultSet(rs) ;    
    } catch ( SQLException e ) {
      return null ;
    }
  }
  
  public static Group loadFromResultSet ( ResultSet rs ) throws SQLException {
    if ( rs.getInt("id") == 0 ) return null ;
    Group g = new Group () ;
    g.setId ( rs.getInt("id") ) ;
    g.setName ( rs.getString("name") ) ;
    g.setCreationDate ( rs.getTimestamp("creation_date") ) ;
    g.setFirstArticle ( rs.getInt("first_article") ) ;
    g.setLastArticle ( rs.getInt("last_article") ) ;
    return g ;
  }
  
  public static LinkedList<Group> getAll () {
    try {
      return loadAllFromStatement ( NNTPConnection.getDB().prepareStatement("SELECT * FROM groups") ) ;
    } catch ( SQLException e ) {
      return null;
    }
  }
  
  public LinkedList<Article> getAllArticles () {
    return getAllArticles ( getFirstArticle(), getLastArticle() ) ;
  }

  public LinkedList<Article> getAllArticles ( int first, int last ) {
    try {
      PreparedStatement stmt = NNTPConnection.getDB().prepareStatement("SELECT * FROM articles, article_groups WHERE article_groups.group_id = ? AND articles.id >= ? AND articles.id <= ? AND articles.id = article_groups.article_id ORDER BY articles.id ASC") ;
      stmt.setInt(1,getId()) ;
      stmt.setInt(2,first) ;
      stmt.setInt(3,last) ;
      return loadAllArticlesFromStatement ( stmt ) ;    
    } catch ( SQLException e ) {
      return null ;
    }
  }
  
  public static LinkedList<Group> getAll ( Timestamp date ) {
    try {
      PreparedStatement stmt = NNTPConnection.getDB().prepareStatement("SELECT * FROM groups WHERE creation_date >= ? ORDER BY name ASC") ;
      stmt.setTimestamp(1,date) ;
      return loadAllFromStatement ( stmt ) ;    
    } catch ( SQLException e ) {
      return null ;
    }
  }

  public static LinkedList<Article> loadAllArticlesFromStatement (PreparedStatement stmt) {
    try {
      ResultSet rs = stmt.executeQuery() ;
      LinkedList<Article> list = new LinkedList<Article> () ;
      while ( rs.next() ) {
        Article g = Article.loadFromResultSet(rs) ;
        if ( g != null ) list.addLast(g) ;
      }
      return list ;
    } catch ( SQLException e ) {
      return null ;
    }
  }

  public static LinkedList<Group> loadAllFromStatement (PreparedStatement stmt) {
    try {
      ResultSet rs = stmt.executeQuery() ;
      LinkedList<Group> list = new LinkedList<Group> () ;
      while ( rs.next() ) {
        Group g = Group.loadFromResultSet(rs) ;
        if ( g != null ) list.addLast(g) ;
      }
      return list ;
    } catch ( SQLException e ) {
      return null ;
    }
  }

  public void update () throws SQLException {
    PreparedStatement stmt = NNTPConnection.getDB().prepareStatement("UPDATE groups SET name = ?, creation_date = ?, first_article = ?, last_article = ?, last_article_date = ? WHERE id = ?;") ;
    stmt.setString(1,getName()) ;
    stmt.setTimestamp(2,getCreationDate());
    stmt.setInt(3,getFirstArticle()) ;
    stmt.setInt(4,getLastArticle()) ;
    stmt.setTimestamp(5,getLastArticleDate()) ;
    stmt.setInt(6,getId()) ;
    stmt.executeUpdate();
  }
  
  public void addArticle ( Article article ) throws SQLException {
      PreparedStatement stmt = NNTPConnection.getDB().prepareStatement("INSERT INTO article_groups (article_id, group_id) VALUES (?, ?);") ;
      stmt.setInt(1,article.getId()) ;
      stmt.setInt(2,getId()) ;
      stmt.execute() ;
      boolean changed = false ; 
      if ( getLastArticleDate() == null || article.getDate().after(getLastArticleDate()) ) {
        setLastArticleDate ( article.getDate() ) ;
        changed = true ;
      }
      if ( article.getId() > getLastArticle() || getLastArticle() == 0 ) {
          setLastArticle ( article.getId() ) ;
          changed = true ;
        }
      if ( article.getId() < getFirstArticle() || getFirstArticle() == 0 ) {
        setFirstArticle ( article.getId() ) ;
        changed = true ;
      }
      if ( changed ) update() ;
  }
  
  public void create () throws SQLException {
    PreparedStatement stmt = NNTPConnection.getDB().prepareStatement("INSERT INTO groups (name,first_article,last_article,creation_date,last_article_date) VALUES (?, ?, ?, ?, ?);") ;
    stmt.setString(1,getName()) ;
    stmt.setInt(2,getFirstArticle()) ;
    stmt.setInt(3,getLastArticle()) ;
    stmt.setTimestamp(4,getCreationDate()) ;
    stmt.setTimestamp(5,getLastArticleDate()) ;
    stmt.execute() ;
    Group g = getByName ( getName() ) ;
    setId(g.getId()) ;
  }
  
  
  /* **************************************************************************
   *                         Getters and Setters
   ****************************************************************************/
  
  
  
  public Timestamp getCreationDate () {
    return creationDate ;
  }
  public void setCreationDate ( Timestamp creationDate ) {
    this.creationDate = creationDate ;
  }
  public int getFirstArticle () {
    return firstArticle ;
  }
  public void setFirstArticle ( int firstArticle ) {
    this.firstArticle = firstArticle ;
  }
  public int getId () {
    return id ;
  }
  public void setId ( int id ) {
    this.id = id ;
  }
  public int getLastArticle () {
    return lastArticle ;
  }
  public void setLastArticle ( int lastArticle ) {
    this.lastArticle = lastArticle ;
  }
  public String getName () {
    return name ;
  }
  public void setName ( String name ) {
    this.name = name ;
  }
  
  public Timestamp getLastArticleDate () {
    return lastArticleDate ;
  }

  public void setLastArticleDate ( Timestamp lastArticleDate ) {
    this.lastArticleDate = lastArticleDate ;
  }

  public int getEstimatedArticleCount () {
    if ( getLastArticle() < getFirstArticle() ) return 0 ;
    return getLastArticle() - getFirstArticle() + 1 ;
  }
  
}
