package tnntpd;

import java.io.IOException;
import java.net.ServerSocket;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;

public class NNTPServer extends Thread {
	
  private ServerSocket socket ;
  public static String hostname ;
  private boolean exit = false ;
  public static NNTPServer instance ;
  
  public NNTPServer ( ServerSocket socket ) throws ClassNotFoundException {
    Class.forName(Main.settings.getProperty("tnntpd.sql.driver"));
    this.socket = socket ;
    hostname = Main.settings.getProperty("tnntpd.hostname") ;
    instance = this ;
  }
  
  public void exit () throws IOException {
    exit = true ;
    socket.close() ;
  }
  
  public void tryInitDB () throws SQLException {
    Connection connection = DriverManager.getConnection(Main.settings.getProperty("tnntpd.sql.database"), Main.settings.getProperty("tnntpd.sql.user"), Main.settings.getProperty("tnntpd.sql.passwd")) ;
    ResultSet rs = connection.getMetaData().getTables(null,null,null,null) ;
    boolean tablesPresent = false ;
    while ( rs.next() ) {
      if ( rs.getString("TABLE_NAME").equalsIgnoreCase("ARTICLES") ) tablesPresent = true ;
      if ( rs.getString("TABLE_NAME").equalsIgnoreCase("GROUPS") ) tablesPresent = true ;
      if ( rs.getString("TABLE_NAME").equalsIgnoreCase("ARTICLE_GROUPS") ) tablesPresent = true ;
      if ( rs.getString("TABLE_NAME").equalsIgnoreCase("HEADERS") ) tablesPresent = true ;
    }
    if ( tablesPresent ) return ;
    connection.nativeSQL("CREATE SEQUENCE SEQ_ARTICLE_ID AS INTEGER START WITH 0" ) ; 
    connection.nativeSQL("CREATE SEQUENCE SEQ_GROUP_ID AS INTEGER START WITH 0" ) ; 
    connection.nativeSQL("CREATE TABLE ARTICLES(ID INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 1)  NOT NULL PRIMARY KEY,MESSAGE_ID VARCHAR(255) NOT NULL,DATE TIMESTAMP NOT NULL,BODY LONGVARCHAR NOT NULL,BODY_TYPE VARCHAR(10) DEFAULT 'text' NOT NULL,CONSTRAINT ARTICLES_MESSAGE_ID UNIQUE(MESSAGE_ID))" ) ;
    connection.nativeSQL("CREATE INDEX ARTICLES_DATE ON ARTICLES(DATE)" ) ;
    connection.nativeSQL("CREATE TABLE HEADERS(ARTICLE_ID INTEGER NOT NULL,NAME VARCHAR(255) NOT NULL,VALUE VARCHAR(255) NOT NULL,CONSTRAINT HEADERS_ID UNIQUE(ARTICLE_ID,NAME),CONSTRAINT HEADERS_ARTICLE_ID FOREIGN KEY(ARTICLE_ID) REFERENCES ARTICLES(ID) ON DELETE CASCADE ON UPDATE CASCADE)" ) ;
    connection.nativeSQL("CREATE TABLE GROUPS(ID INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 1)  NOT NULL PRIMARY KEY,NAME VARCHAR(255) NOT NULL,CREATION_DATE TIMESTAMP NOT NULL,FIRST_ARTICLE INTEGER,LAST_ARTICLE INTEGER,LAST_ARTICLE_DATE TIMESTAMP,CONSTRAINT GROUP_NAME UNIQUE(NAME))" ) ;
    connection.nativeSQL("CREATE INDEX GROUPS_CREATION_DATE ON GROUPS(CREATION_DATE)" ) ;
    connection.nativeSQL("CREATE TABLE ARTICLE_GROUPS(ARTICLE_ID INTEGER NOT NULL,GROUP_ID INTEGER NOT NULL,CONSTRAINT ARTICLE_GROUPS_ID UNIQUE(ARTICLE_ID,GROUP_ID),CONSTRAINT AG_ARTICLE_ID FOREIGN KEY(ARTICLE_ID) REFERENCES ARTICLES(ID) ON DELETE CASCADE ON UPDATE CASCADE,CONSTRAINT AG_GROUP_ID FOREIGN KEY(GROUP_ID) REFERENCES GROUPS(ID) ON DELETE CASCADE ON UPDATE CASCADE)" ) ;
    connection.nativeSQL("CREATE SEQUENCE SEQ_ARTICLE_ID AS INTEGER START WITH 0" ) ;
    connection.nativeSQL("CREATE USER SA PASSWORD \"\" ADMIN" ) ;
    connection.nativeSQL("SET WRITE_DELAY 60" ) ;
    connection.commit() ;
    connection.close() ;
  }
  
  public void run () {
    if ( ! this.equals(Thread.currentThread()) ) throw new RuntimeException ( "Do not call run() directly !" ) ;
    if ( Main.settings.getProperty("tnntpd.tryinitdb") != null ) try {
      tryInitDB () ;
    } catch ( Throwable e1 ) {
      e1.printStackTrace();
    }
    while ( ! exit ) {
      try {
        new NNTPConnection ( socket.accept(), DriverManager.getConnection(Main.settings.getProperty("tnntpd.sql.database"), Main.settings.getProperty("tnntpd.sql.user"), Main.settings.getProperty("tnntpd.sql.passwd")) ).start () ;
      } catch ( IOException e ) {
        if ( exit ) return ;
        e.printStackTrace();
      } catch ( SQLException e ) {
        e.printStackTrace();
      }
    }
  }
  
}
