package tnntpd.command;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.util.zip.GZIPInputStream;

import tnntpd.Article;
import tnntpd.NNTPConnection;

public class ArticleCommand extends Command {

  public static boolean process ( String[] command ) throws IOException {
    String commandName = command[0] ;
    if ( ! ( commandName.equalsIgnoreCase("ARTICLE") || commandName.equalsIgnoreCase("STAT") || commandName.equalsIgnoreCase("HEAD") || commandName.equalsIgnoreCase("BODY") ) ) return false ;
    // untested, RFC977 complient
    Article article = null;
    if ( command.length <= 1 ) {
      article = getCurrentArticle() ;
      if ( article == null ) {
        printStatus ( 420, "no current article has been selected") ;
        return true;
      }
    } else if ( command[1].matches(NNTPConnection.MESSAGE_ID_PATTERN) ) {
      // Message-ID 
      article = Article.getByMessageId ( command[1] ) ;
      if ( article == null ) {
        printStatus ( 430, "no such article found") ;
        return true;
      }
    } else {
      // Message Number
      try {
        int num = Integer.parseInt ( command[1] ) ;
        article = Article.getById ( num ) ;
      } catch ( Exception e ) {}
      if ( article == null ) {
        printStatus ( 423, "no such article number in this group") ;
        return true;
      }
      setCurrentArticle ( article ) ;
    }
    BufferedReader body ;
    if ( article.getBodyType().equalsIgnoreCase(Article.BODY_TYPE_FILE) ) {
      File store = new File ( getArticleDir(), article.getBody() ) ;
      body = new BufferedReader ( new InputStreamReader ( new FileInputStream ( store ) ) ) ;
    } else if ( article.getBodyType().equalsIgnoreCase(Article.BODY_TYPE_FILE_GZIP) ) {
      File store = new File ( getArticleDir(), article.getBody() ) ;
      body = new BufferedReader ( new InputStreamReader ( new GZIPInputStream ( new FileInputStream ( store ) ) ) ) ;
    } else {
      body = new BufferedReader ( new StringReader ( article.getBody() ) ) ;
    }
    if ( commandName.equalsIgnoreCase("ARTICLE") ) {
      printStatus ( 220, article.getId() + " " + article.getMessageId() + " article retrieved - head and body follow") ;
      printTextPart ( article.getCompleteHeader().toString() ) ;
      println("") ;
      printText ( body ) ;
    }
    if ( commandName.equalsIgnoreCase("HEAD") ) {
      printStatus ( 221, article.getId() + " " + article.getMessageId() + " article retrieved - head follows") ;
      printText ( article.getCompleteHeader().toString() ) ;
    }
    if ( commandName.equalsIgnoreCase("BODY") ) {
      printStatus ( 222, article.getId() + " " + article.getMessageId() + " article retrieved - body follows") ;
      printText ( body ) ;
    }
    if ( commandName.equalsIgnoreCase("STAT") ) {
      printStatus ( 223, article.getId() + " " + article.getMessageId() + " article retrieved - request text separately") ;
    }
    return true ;
  }
  
}
